/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│vi: set net ft=c ts=2 sts=2 sw=2 fenc=utf-8                                :vi│
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2022 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ Permission to use, copy, modify, and/or distribute this software for         │
│ any purpose with or without fee is hereby granted, provided that the         │
│ above copyright notice and this permission notice appear in all copies.      │
│                                                                              │
│ THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL                │
│ WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED                │
│ WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE             │
│ AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL         │
│ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        │
│ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER               │
│ TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR             │
│ PERFORMANCE OF THIS SOFTWARE.                                                │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/calls/strace.internal.h"
#include "libc/calls/syscall_support-nt.internal.h"
#include "libc/intrin/describeflags.internal.h"
#include "libc/nt/memory.h"
#include "libc/nt/process.h"
#include "libc/nt/struct/securityattributes.h"
#include "libc/nt/thunk/msabi.h"

__msabi extern typeof(OpenProcess) *const __imp_OpenProcess;

/**
 * Creates file mapping object on the New Technology.
 *
 * @param dwDesiredAccess should be kNtProcess... combination
 * @return ehandle, or 0 on failure
 * @note this wrapper takes care of ABI, STRACE(), and __winerr()
 * @see MapViewOfFileEx()
 */
textwindows int64_t OpenProcess(uint32_t dwDesiredAccess, bool32 bInheritHandle,
                                uint32_t dwProcessId) {
  int64_t hHandle;
  hHandle = __imp_OpenProcess(dwDesiredAccess, bInheritHandle, dwProcessId);
  if (!hHandle) __winerr();
  NTTRACE("OpenProcess(%s, %hhhd, %u) → %ld% m",
          DescribeNtProcessAccessFlags(dwDesiredAccess), bInheritHandle,
          dwProcessId, hHandle);
  return hHandle;
}
